#!/usr/bin/env python3

from pydoc import cli
import grpc
import logging


from pbes import sequense_pb2
from pbes import sequense_pb2_grpc



def client():
    with grpc.insecure_channel("127.0.0.1:10352") as channel:
        stub = sequense_pb2_grpc.SequenceServerStub(channel)
        response = stub.get(sequense_pb2.SequenceRequest())
        print(response.sequence)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sequense.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='sequense.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0esequense.proto\"1\n\x0fSequenceRequest\x12\x13\n\x06offset\x18\x01 \x01(\x03H\x00\x88\x01\x01\x42\t\n\x07_offset\"$\n\x10SequenceResponse\x12\x10\n\x08sequence\x18\x01 \x01(\x03\x32>\n\x0eSequenceServer\x12,\n\x03get\x12\x10.SequenceRequest\x1a\x11.SequenceResponse\"\x00\x62\x06proto3'
)




_SEQUENCEREQUEST = _descriptor.Descriptor(
  name='SequenceRequest',
  full_name='SequenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='offset', full_name='SequenceRequest.offset', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='_offset', full_name='SequenceRequest._offset',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=18,
  serialized_end=67,
)


_SEQUENCERESPONSE = _descriptor.Descriptor(
  name='SequenceResponse',
  full_name='SequenceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence', full_name='SequenceResponse.sequence', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=69,
  serialized_end=105,
)

_SEQUENCEREQUEST.oneofs_by_name['_offset'].fields.append(
  _SEQUENCEREQUEST.fields_by_name['offset'])
_SEQUENCEREQUEST.fields_by_name['offset'].containing_oneof = _SEQUENCEREQUEST.oneofs_by_name['_offset']
DESCRIPTOR.message_types_by_name['SequenceRequest'] = _SEQUENCEREQUEST
DESCRIPTOR.message_types_by_name['SequenceResponse'] = _SEQUENCERESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SequenceRequest = _reflection.GeneratedProtocolMessageType('SequenceRequest', (_message.Message,), {
  'DESCRIPTOR' : _SEQUENCEREQUEST,
  '__module__' : 'sequense_pb2'
  # @@protoc_insertion_point(class_scope:SequenceRequest)
  })
_sym_db.RegisterMessage(SequenceRequest)

SequenceResponse = _reflection.GeneratedProtocolMessageType('SequenceResponse', (_message.Message,), {
  'DESCRIPTOR' : _SEQUENCERESPONSE,
  '__module__' : 'sequense_pb2'
  # @@protoc_insertion_point(class_scope:SequenceResponse)
  })
_sym_db.RegisterMessage(SequenceResponse)



_SEQUENCESERVER = _descriptor.ServiceDescriptor(
  name='SequenceServer',
  full_name='SequenceServer',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=107,
  serialized_end=169,
  methods=[
  _descriptor.MethodDescriptor(
    name='get',
    full_name='SequenceServer.get',
    index=0,
    containing_service=None,
    input_type=_SEQUENCEREQUEST,
    output_type=_SEQUENCERESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SEQUENCESERVER)

DESCRIPTOR.services_by_name['SequenceServer'] = _SEQUENCESERVER

# @@protoc_insertion_point(module_scope)



import os
import re
from setuptools import setup

setup(name='seqence-server-demo',
      version='0.0.1',
      description='seqence-server-demo',
      author="Neeky",
      author_email="neeky@live.com",
      maintainer='Neeky',
      maintainer_email='neeky@live.com',
      scripts=['bin/sequence-client','bin/sequence-server'],
      packages=['pbes'],
      url='https://github.com/Neeky/protobuf-es',
      install_requires=['grpcio==1.39.0','grpcio-tools==1.38.1','protobuf==3.19.3'],
      python_requires='>=3.8.*',
      classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'Operating System :: POSIX',
          'Operating System :: MacOS :: MacOS X',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8']
      )


#!/usr/bin/env python3

"""
protobuf 搭配 grpc 来实现序列号服务器
"""

import sys
import grpc
import logging
import argparse
from concurrent import futures
from threading import RLock

from pbes import sequense_pb2
from pbes import sequense_pb2_grpc

logging.basicConfig(level=logging.DEBUG,format="%(asctime)s - %(name)s - %(threadName)s - %(levelname)s - %(lineno)s - %(message)s")

name = "sequence-server-impl-by-python-3"

def parse():
    parser = argparse.ArgumentParser(name)
    parser.add_argument("--ip",type=str,default="127.0.0.1")
    parser.add_argument("--port",type=int,default=10352)
    parser.add_argument("action",type=str,default="exit",choices=["exit","run"])
    return parser.parse_args()

class SeqenceServer(sequense_pb2_grpc.SequenceServer):
    current = 0
    lock = RLock()
    def get(self,request,context):
        """
        """
        logging.info(f"revice request from {context.peer()} offset = {request.offset} .")
        logging.info(f"current = {self.current} .")
        with self.lock:
            self.current = self.current + (request.offset if request.offset != 0 else 1)
            return sequense_pb2.SequenceResponse(sequence=self.current)

def serve():
    # 处理请求参数
    args = parse()
    if args.action == "exit":
        logging.warning("exit bye .")
        sys.exit(0)
    
    host = args.ip
    port = args.port

    logging.info(f"sequence-server start listen on {host}:{port} .")
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    sequense_pb2_grpc.add_SequenceServerServicer_to_server(SeqenceServer(), server)
    server.add_insecure_port(f'{host}:{port}')
    server.start()
    logging.info("seqense-server started .")
    server.wait_for_termination()
